
#include "spritelib.h"

/** SOIL, used to read texture images in spritelib::read_tex **/
#include "../soil/SOIL.h" /* Simple OpenGL Image Library, www.lonesock.net/soil.html (plus Dr. Lawlor's modifications) */
#include "../soil/SOIL.c" /* just slap in implementation files here, for easier linking */
#include "../soil/stb_image_aug.c"

#include <iostream>
#include <cmath>
using std::cos; using std::sin;


/* Return a 2D location, equal to (a,b) rotated by this angle around (x,y) */
const float *rotate_2D(float angle,float x,float y,float a,float b)
{
	float d2r=(float)(3.14159265358979/180.0);
	float c=cos(angle*d2r), s=sin(angle*d2r);
	static float ret[2];
	ret[0]=x+a*c+b*s;
	ret[1]=y-a*s+b*c;
	return ret;
}

/* Draw a square sprite with this texture, 
   centered at pixel (x,y), 
   of size w by h pixels,
   rotated counterclockwise by angle "angle" (in degrees)
*/
void spritelib::draw(spritelib_tex tex, 
	float x,float y, float w,float h,
	float angle,bool glow) 
{
	if (glow) glBlendFunc(GL_ONE,GL_ONE); /* just add everything together */
	else glBlendFunc(GL_ONE,GL_ONE_MINUS_SRC_ALPHA); /* erase previous stuff */
	glBindTexture(GL_TEXTURE_2D,tex);
	glEnable(GL_TEXTURE_2D);
	glBegin(GL_TRIANGLE_FAN);
	w*=0.5f, h*=0.5f; // half width and height
	glTexCoord2f(0.0f,1.0f); glVertex2fv(rotate_2D(angle,x,y,-w,-h));
	glTexCoord2f(1.0f,1.0f); glVertex2fv(rotate_2D(angle,x,y,+w,-h));
	glTexCoord2f(1.0f,0.0f); glVertex2fv(rotate_2D(angle,x,y,+w,+h));
	glTexCoord2f(0.0f,0.0f); glVertex2fv(rotate_2D(angle,x,y,-w,+h));
	glEnd();
}

static bool spritelib_initted=false;
spritelib_tex spritelib::read_tex(const char *filename)
{
	return SOIL_load_OGL_texture
        (
                filename,
                SOIL_LOAD_AUTO,
                SOIL_CREATE_NEW_ID,
                SOIL_FLAG_MIPMAPS | SOIL_FLAG_INVERT_Y,
                GL_RGBA8
        );
}

/* Return the GUI's current spritelib object */
spritelib &spritelib_get(bool check_modifiers) {
	static spritelib lib;
	if (check_modifiers) {
		int mod=glutGetModifiers();
		lib.shift=(mod&GLUT_ACTIVE_SHIFT)!=0;
		lib.ctrl =(mod&GLUT_ACTIVE_CTRL)!=0;
		lib.alt  =(mod&GLUT_ACTIVE_ALT)!=0;
	}
	return lib;
}

/* GLUT display function */
void spritelib_display(void) 
{
	spritelib &lib=spritelib_get(false);
	glDisable(GL_DEPTH_TEST);
	// Dunno if alpha is actually a good idea or not...
	glBlendFunc(GL_ONE,GL_ONE_MINUS_SRC_ALPHA); // premultiplied alpha
	glEnable(GL_BLEND);
	
	glClearColor(lib.background[0],lib.background[1],
		lib.background[2],lib.background[3]); // background color
	glClear(GL_COLOR_BUFFER_BIT + GL_DEPTH_BUFFER_BIT);
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity(); // flush any ancient matrices
	
	lib.screenw=glutGet(GLUT_WINDOW_WIDTH);
	lib.screenh=glutGet(GLUT_WINDOW_HEIGHT);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity(); // flush any ancient matrices
	glTranslatef(-1.0f,+1.0f,0.0f);
	glScalef(2.0/lib.screenw,-2.0/lib.screenh,0.0);
	static float oldtime=0.0;
	float newtime=glutGet(GLUT_ELAPSED_TIME)*0.001;
	lib.dt=newtime-oldtime; if (lib.dt>1.0) lib.dt=1.0;
	lib.time=newtime;
	oldtime=newtime;
	
	glBindTexture(GL_TEXTURE_2D,0); /* turn off texture */
	glColor3f(1.0,1.0,1.0); /* white */
	
	/* Call user's drawing function */
	spritelib_draw_screen(lib);
	
	/* Clear all the oneshots */
	for (int key=0;key<256;key++) lib.key_typed[key]=false;
	lib.mouseleft_clicked=false;
	lib.mouseright_clicked=false;
	
	glutPostRedisplay(); // continual animation
	glutSwapBuffers();
}



void spritelib_mouse(int button,int state,int x,int y) 
{ /* mouse being pressed or released--save position for motion */
	spritelib &lib=spritelib_get();
	if (state==GLUT_DOWN) {
		if (button==0) {lib.mouseleft=true; lib.mouseleft_clicked=true;}
		else if (button==2) {lib.mouseright=true; lib.mouseright_clicked=true;}
		else if (button==3) {lib.mousewheel++;}
		else if (button==4) {lib.mousewheel--;}
	} else {
		if (button==0) {lib.mouseleft=false;}
		else if (button==2) {lib.mouseright=false;}
	}
	lib.mousex=x; lib.mousey=y;
	glutPostRedisplay(); // start redisplays immediately
	
}
void spritelib_motion(int x, int y) { 
	spritelib &lib=spritelib_get(false);
	lib.mousex=x; lib.mousey=y;
	glutPostRedisplay(); // start redisplays immediately
}
void spritelib_keyboard(unsigned char key,int x,int y) {
	spritelib &lib=spritelib_get();
	if (!lib.key[key]) lib.key_typed[key]=true; /* first time! */
	lib.key[key]=true;
	glutPostRedisplay();
}
void spritelib_keyboardUp(unsigned char key,int x,int y) {
	spritelib &lib=spritelib_get();
	lib.key[key]=false;
	glutPostRedisplay();
}

/** You must call this function once, from your main function.
  Spritelib will open a window with the given title and size,
  and then handle events to that window until you exit. 
  This function never returns. */
void spritelib_run(const char *window_title,int screenw,int screenh)
{
	int argc=0; char **argv=0;
	glutInit(&argc,argv);
	glutInitDisplayMode(GLUT_RGBA + GLUT_DEPTH + GLUT_DOUBLE);
	glutInitWindowSize(screenw,screenh);
	glutCreateWindow(window_title);
	
	glutDisplayFunc(spritelib_display);
	glutMouseFunc(spritelib_mouse);
	glutMotionFunc(spritelib_motion); 
	glutPassiveMotionFunc(spritelib_motion); 
	glutKeyboardFunc(spritelib_keyboard);
	glutKeyboardUpFunc(spritelib_keyboardUp);
	
	glutMainLoop();
}
